#include <stdio.h>
#include <math.h>

#include "video_quality.h"

/* From "A Universal Quality Index", Zhou Wang and Alan C. Bovik
 * Q = (correlation coefficient) (mean difference) (contrast difference)
 *   = (Var(x,y) / sqrt(Var(x) Var(y)))
 *     (2 mean(x) mean(y) / (mean(x)² + mean(y)²))
 *     (2 sqrt(Var(x) Var(y)) / (Var(x) + Var(y)))
 * = 4 Var(x,y) mean(x) mean(y) / ((Var(x) + Var(y)) (mean(x)² + mean(y)²))
 * = (N[xy] - [x][y]) [x] [y] / ((N[x²]-[x]²+N[y²]-[y]²) ([x]²+[y]²))
 * [foo] = sum(foo)
 */

void
vq_correl_init(vq_correl_t *p)
{
    p->n = 0;
    p->x = 0;
    p->y = 0;
    p->xx = 0;
    p->yy = 0;
    p->xy = 0;
}

void
vq_correl_feed(vq_correl_t *p, const unsigned char *d1,
    const unsigned char *d2, unsigned n)
{
    statistic_int_t x = 0, y = 0, xx = 0, yy = 0, xy = 0;
    unsigned i;

    for(i = 0; i < n; i++) {
	x += *d1;
	y += *d2;
	xx += *d1 * *d1;
	yy += *d2 * *d2;
	xy += *d1 * *d2;
	d1++;
	d2++;
    }
    p->n += n;
    p->x += x;
    p->y += y;
    p->xx += xx;
    p->yy += yy;
    p->xy += xy;
}

void
vq_correl_print(const vq_correl_t *p, const char *t)
{
    double v;
    double n, sx, sy, sxx, syy, sxy;

    n = p->n;
    sx = p->x;
    sy = p->y;
    sxx = p->xx;
    syy = p->yy;
    sxy = p->xy;
    v = 4 * (n * sxy - sx * sy ) * sx * sy /
	((n * sxx - sx * sx + n * syy - sy * sy) * (sx * sx + sy * sy));
    printf("corr%s = %.12f\n", t, v);
}

void
vq_correl_add(vq_correl_t *a, unsigned ka, const vq_correl_t * b, unsigned kb)
{
    a->n = ka * a->n + kb * b->n;
    a->x = ka * a->x + kb * b->x;
    a->y = ka * a->y + kb * b->y;
    a->xx = ka * a->xx + kb * b->xx;
    a->yy = ka * a->yy + kb * b->yy;
    a->xy = ka * a->xy + kb * b->xy;
}
