#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <gdk-pixbuf/gdk-pixbuf.h>

#include "video_quality.h"

unsigned width = 0;
unsigned height = 0;

static GdkPixbuf *
load_file(const char *filename)
{
    GError *err = NULL;
    GdkPixbuf *p;

    if(width == 0)
	p = gdk_pixbuf_new_from_file(filename, &err);
    else
	p = gdk_pixbuf_new_from_file_at_size(filename, width, height, &err);
    if(p == NULL) {
	fprintf(stderr, "Loading %s: %s\n", filename, err->message);
	exit(1);
    }
    return(p);
}

int
main(int argc, char **argv)
{
    GdkPixbuf *p1, *p2;
    unsigned c1, c2;
    unsigned s1, s2;
    unsigned char *d1, *d2;
    unsigned y;
    vq_psnr_t psnr;
    vq_correl_t corr;

    g_type_init();
    argc--;
    argv++;
    if(argc != 2) {
	printf("Usage: image_quality [options] image1 image2\n");
	exit(1);
    }
    p1 = load_file(argv[0]);
    if(width == 0) {
	width = gdk_pixbuf_get_width(p1);
	height = gdk_pixbuf_get_height(p1);
    }
    p2 = load_file(argv[1]);
    c1 = gdk_pixbuf_get_n_channels(p1);
    c2 = gdk_pixbuf_get_n_channels(p2);
    if(c1 != c2) {
	fprintf(stderr, "Not the same format\n");
	exit(1);
    }
    s1 = gdk_pixbuf_get_rowstride(p1);
    s2 = gdk_pixbuf_get_rowstride(p2);
    d1 = gdk_pixbuf_get_pixels(p1);
    d2 = gdk_pixbuf_get_pixels(p2);
    vq_psnr_init(&psnr);
    vq_correl_init(&corr);
    for(y = 0; y < height; y++) {
	vq_psnr_feed(&psnr, d1, d2, c1 * width);
	vq_correl_feed(&corr, d1, d2, c1 * width);
	d1 += s1;
	d2 += s2;
    }
    vq_psnr_print(&psnr, "");
    vq_correl_print(&corr, "");
    return(0);
}
